#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2017-02-07 19:25 manu>

import sqlite3
import logging
import datetime
import json
import time
import sys
import os
import re

import emoji

from lib.orgproperty import OrgProperties
from lib.orgformat import OrgFormat
from lib.memacs import Memacs


class WhatsApp(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--file", dest="msgstore",
            action="store", type=file, required=True,
            help="path to decrypted msgstore.db file")

        self._parser.add_argument(
            "--ignore-incoming", dest="ignore_incoming",
            action="store_true", help="ignore received messages")

        self._parser.add_argument(
            "--ignore-outgoing", dest="ignore_outgoing",
            action="store_true", help="ignore sent messages")

        self._parser.add_argument(
            "--output-format", dest="output_format",
            action="store", default="{verb} [[{handler}:{number}][{number}]]: {text}",
            help="format string to use for the headline")

        self._parser.add_argument(
            "--set-handler", dest="handler",
            action="store", default="tel",
            help="set link handler")

        self._parser.add_argument(
            "--demojize", dest="demojize",
            action="store_true", help="replace emoji with the appropriate :shortcode:")

        self._parser.add_argument(
            "--skip-emoji", dest="skip_emoji",
            action="store_true", help="skip all emoji")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

    def _is_ignored(self, msg):
        """check for ignored message type"""

        if msg['type'] is 'INCOMING' and self._args.ignore_incoming:
            return True

        if msg['type'] is 'OUTGOING' and self._args.ignore_outgoing:
            return True

    def _handle_message(self, msg):
        """parse a single message row"""

        msg['number'] = '00' + msg['number'].split('@')[0]
        msg['verb'] = 'to' if msg['type'] else 'from'
        msg['type'] = 'OUTGOING' if msg['type'] else 'INCOMING'
        msg['handler'] = self._args.handler

        if msg['text']:
            if self._args.demojize:
                msg['text'] = emoji.demojize(msg['text'])

            if self._args.skip_emoji:
                msg['text'] = re.sub(emoji.get_emoji_regexp(), '', msg['text'])

        timestamp = datetime.datetime.fromtimestamp(msg['timestamp'] / 1000)

        properties = OrgProperties(data_for_hashing=json.dumps(msg))
        properties.add('NUMBER', msg['number'])
        properties.add('TYPE', msg['type'])

        output = self._args.output_format.decode('utf-8').format(**msg)

        if msg['text'] and not self._is_ignored(msg):
            self._writer.write_org_subitem(timestamp=OrgFormat.datetime(timestamp),
                                           output=output, properties=properties)

    def _main(self):
        """
        get's automatically called from Memacs class
        """

        conn = sqlite3.connect(os.path.abspath(self._args.msgstore.name))
        query = conn.execute('SELECT * FROM messages')

        for row in query:
            self._handle_message({
                'timestamp': row[7],
                'number': row[1],
                'type': row[2],
                'text': row[6]
            })

            logging.debug(row)



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import subprocess
from datetime import datetime
from lib.orgproperty import OrgProperties
from lib.orgformat import OrgFormat
from lib.memacs import Memacs
import re

class MuMail(Memacs):
        
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
           "-q", "--query",
           dest="query",
           help="mu search query")

        self._parser.add_argument(
           "-m", "--me",
           dest="sender",
           help="space seperated list of mail addresses that belongs to you")
        
        self._parser.add_argument(
            "-d", "--delegation",
            dest="todo",
            action='store_true',
            help="adds NEXT or WAITING state to flagged messages")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs
        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

        self._query = []

        if self._args.sender:
            self._args.sendern = self._args.sender.strip()
            self._sender = list(self._args.sender.split(" "))
        else:
            raise ValueError('You have to specify at least one e mail adress')

        if self._args.query:
            self._query = self._args.query

        if self._args.todo:
            self._todo = True
        else:
            self._todo = False

    def __parse_Plain(self,plain_mails):
        messages = plain_mails.decode('utf-8')
        return messages.splitlines()

    def __getTimestamp(self, time, onlyDate=False):
        """
        converts xml timestamp into org readable timestamp
        Do  6 Nov 21:22:17 2014
        """
        time = time.strip().encode('utf-8')

        mail_date = datetime.strptime(time,"%c")
        if onlyDate is False:
            return OrgFormat.datetime(mail_date)
        return OrgFormat.date(mail_date)

    def __create_mail_link(self, sender):
        """
        creates well formated org mail link from message 'from' field.
        """
        rex = re.compile('([\w\s.,-]*?)[\s<"]*([\w.-]+@[\w.-]+)',re.UNICODE)
        m = rex.search(sender)
        if m:
            name = m.group(1).strip()
            mail = m.group(2).strip()
            if name is not u"":
                return (u"[[mailto:" + mail + u"][" + name + u"]]",name,mail)
            else:
                return (u"[[mailto:" + mail + u"][" + mail + u"]]",name,mail)
        return (u"Unknown",u"Unknown",u"Unknown")
        
        
    def _main(self):
        """
        get's automatically called from Memacs class
        fetches all mails out of mu database
        """
        command = self._query
        # command.extend(self._query)
        command = command+" --fields=t:#:d:#:f:#:g:#:s:#:i --format=plain"
        try:
            xml_mails = subprocess.check_output(command, shell=True)
        except:
            print("something goes wrong")
            exit()
        messages = self.__parse_Plain(xml_mails)

        properties = OrgProperties()
        for message in messages:
            (an,datum,von,flags,betreff,msgid) = message.split(":#:")
            betreff = betreff.replace("[","<")
            betreff = betreff.replace("]",">")
            properties.add('TO',an)
            if von != "":
                (sender,vname,vmail) = self.__create_mail_link(von)
                (an,aname,amail) = self.__create_mail_link(an)
                timestamp = self.__getTimestamp(datum)
                properties.add_data_for_hashing(timestamp + "_" + msgid)
                properties.add("FROM",sender)
                notes = ""
                if any(match in vmail for match in self._sender):
                    output = output = "".join(["T: ",an,": [[mu4e:msgid:",msgid,"][",betreff,"]]"])
                    pre = 'WAITING '
                else:
                    output = "".join(["F: ",sender,": [[mu4e:msgid:",msgid,"][",betreff,"]]"])
                    pre = 'NEXT '
                if (flags.find('F') >= 0 and self._todo):
                    date = self.__getTimestamp(datum,True)
                    notes = "SCHEDULED: "+date
                    timestamp = ""
                    output = pre+output
            self._writer.write_org_subitem(timestamp, output, notes, properties)



# -*- coding: utf-8 -*-
# Time-stamp: <2012-03-09 15:36:52 armin>

import os
import re
import time
import unittest
import xml.etree.ElementTree as ET

from memacs.sms import SmsMemacs


class TestSmsMemacs(unittest.TestCase):

    def setUp(self):
        self._test_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'smsxml.txt'
        )
        argv = "-s -f " + self._test_file
        memacs = SmsMemacs(argv=argv.split())
        self.data = memacs.test_get_entries()
        self._smses = ET.parse(self._test_file)

    def test_from_file(self):
        data = self.data

        for i in range(4):
            self._assertSMSLog(i, data[i*6:(i+1)*6])

    def _assertSMSLog(self, index, sms_data):
        sms = self._smses.findall('sms')[index]

        name = sms_data[3].split()[-1]
        name = 'Unknown' if name == 'False' else name
        to_from = 'SMS to' if sms.get('type') == '2' else 'SMS from'
        number = sms_data[2].split()[-1]
        timestamp, = re.findall('<(.*?)>', sms_data[0])
        date = time.gmtime(int(sms.get('date'))/1000.)

        self.assertIn(name, sms_data[0])
        self.assertIn(to_from, sms_data[0])
        self.assertEqual(time.strftime('%Y-%m-%d %a %H:%M', date), timestamp)
        self.assertEqual(number, sms.get('address').replace('+',u'00'))



# -*- coding: utf-8 -*-

import unittest
import os

from memacs.battery import Battery


class TestCsv(unittest.TestCase):

    def test_battery(self):

        path = os.path.dirname(os.path.abspath(__file__))

        argv = []
        argv.append("-p")
        argv.append(os.path.join(path, "data"))

        memacs = Battery(argv=argv)
        data = memacs.test_get_entries()

        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(data[2], "   :STATUS:      discharging")
        self.assertEqual(data[3], "   :CAPACITY:    97%")
        self.assertEqual(data[4], "   :CYCLE_COUNT: 866")
        self.assertEqual(data[5], "   :CONSUMPTION: 11.9 W")
        self.assertEqual(data[7], "   :END:")


#!/usr/bin/python2
# -*- coding: utf-8 -*-

import time
import datetime
import logging
import sys

import pylast

from lib.orgproperty import OrgProperties
from lib.orgformat import OrgFormat
from lib.memacs import Memacs


class LastFM(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            '--output-format', dest='output_format',
            action='store', default=u'{title}',
            help='formt string to use for the output'
        )

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

        if self._args.output_format:
            self._args.output_format = self._args.output_format.decode('utf-8')

    def _handle_recent_tracks(self, tracks):
        """parse recent tracks"""
        logging.debug(tracks)

        for t in tracks:
            timestamp = datetime.datetime.fromtimestamp(int(t.timestamp))

            output = self._args.output_format.format(title=t.track.title,
                                                     artist=t.track.artist,
                                                     album=t.album)

            properties = OrgProperties(data_for_hashing=t.timestamp)
            properties.add('ARTIST', t.track.artist)
            properties.add('ALBUM', t.album)

            self._writer.write_org_subitem(timestamp=OrgFormat.datetime(timestamp),
                                           output=output,
                                           properties=properties)

    def _main(self):
        """
        get's automatically called from Memacs class
        """

        options = {
            'api_secret': self._get_config_option('api_secret'),
            'api_key': self._get_config_option('api_key'),
            'password_hash': pylast.md5(self._get_config_option('password')),
            'username': self._get_config_option('username')
        }

        try:

            if 'lastfm' in self._get_config_option('network'):
                network = pylast.get_lastfm_network(**options)

            if 'librefm' in self._get_config_option('network'):
                network = pylast.get_librefm_network(**options)

            user = network.get_user(options['username'])

            self._handle_recent_tracks(user.get_recent_tracks(limit=100))

        except pylast.WSError, e:
            logging.error('an issue with the network web service occured: %s' % e)
            sys.exit(1)



# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31>

import os
import time
import unittest

from memacs.whatsapp import WhatsApp


class TestWhatsApp(unittest.TestCase):

    def setUp(self):
        msgstore = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'msgstore.db'
        )

        self.argv = []
        self.argv.append('-f')
        self.argv.append(msgstore)

    def test_whatsapp(self):
        self.argv.append('--output-format')
        self.argv.append('{text}')

        memacs = WhatsApp(argv=self.argv)
        data = memacs.test_get_entries()

        self.assertEqual(data[0], '** <2016-10-15 Sat 20:18> Hello World!')
        self.assertEqual(data[1], '   :PROPERTIES:')
        self.assertEqual(data[2], '   :TYPE:       INCOMING')
        self.assertEqual(data[3], '   :NUMBER:     00436604444333')
        self.assertEqual(data[4], '   :ID:         d8fa64a86286432a42a13f129cad0d7ebd56873e')
        self.assertEqual(data[5], '   :END:')



# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31 aw>

import unittest
import os
from memacs.rss import RssMemacs


class TestRss(unittest.TestCase):

    def setUp(self):
        self.test_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'data', 'sample-rss.txt'
        )
        self.argv = "-s -f " + self.test_file

    def test_false_appending(self):
        try:
            memacs = RssMemacs(argv=self.argv.split())
            memacs.test_get_entries()
        except Exception:
            pass

    def test_all(self):
        memacs = RssMemacs(argv=self.argv.split())
        data = memacs.test_get_entries()

        self.assertEqual(
            data[0],
            (
                "** <2009-09-06 Sun 16:45> "
                "[[http://www.wikipedia.org/][Example entry]]"
            )
        )
        self.assertEqual(
            data[1],
            "   Here is some text containing an interesting description."
        )
        self.assertEqual(
            data[3],
            "   :GUID:       unique string per item")
        self.assertEqual(
            data[4],
            '   :PUBLISHED:  Mon, 06 Sep 2009 16:45:00 +0000'
        )
        self.assertEqual(
            data[5],
            "   :ID:         a0df7d405a7e9822fdd86af04e162663f1dccf11"
        )

        self.assertEqual(data[2], "   :PROPERTIES:")
        self.assertEqual(data[6], "   :END:")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2017-04-13 22:19:39 vk>

import datetime
import logging
import os
import re
import time

from lib.orgformat import OrgFormat
from lib.memacs import Memacs
from lib.reader import CommonReader
from lib.orgproperty import OrgProperties


class SimplePhoneLogsMemacs(Memacs):

    _REGEX_SEPARATOR = " *?# *?"

    ## match for example: "2012-11-20 # 19.59 # shutdown #   72 # 35682"
    ##                     0            1  2    3            4    5
    LOGFILEENTRY_REGEX = re.compile("([12]\d\d\d-[012345]\d-[012345]\d)" +
                                    _REGEX_SEPARATOR +
                                    "([ 012]\d)[:.]([012345]\d)" +
                                    _REGEX_SEPARATOR +
                                    "(.+)" +
                                    _REGEX_SEPARATOR +
                                    "(\d+)" +
                                    _REGEX_SEPARATOR +
                                    "(\d+)$", flags=re.U)
    RE_ID_DATESTAMP = 0
    RE_ID_HOURS = 1
    RE_ID_MINUTES = 2
    RE_ID_NAME = 3
    RE_ID_BATT = 4
    RE_ID_UPTIME = 5

    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--file", dest="phonelogfile",
            action="store", required=True,
            help="path to phone log file")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)
        if not (os.path.exists(self._args.phonelogfile) or
                os.access(self._args.phonelogfile, os.R_OK)):
            self._parser.error("input file not found or not readable")

    def _generateOrgentry(self, e_time, e_name, e_batt, e_uptime,
                          e_last_opposite_occurrence, e_last_occurrence,
                          prev_office_sum, prev_office_first_begin, office_lunchbreak,
                          battery_percentage_when_booting):
        """
        takes the data from the parameters and generates an Org-mode entry.

        @param e_time: time-stamp of the entry
        @param e_name: entry name/description
        @param e_batt: battery level
        @param e_uptime: uptime in seconds
        @param e_last_opposite_occurrence: time-stamp of previous opposite occurrence (if not False)
        @param e_last_occurrence: time-stamp of previous occurrence
        @param additional_paren_string: string that gets appended to the parenthesis
        @param prev_office_sum: holds the sum of all previous working duration today
        @param prev_office_first_begin: holds the first time-stamp of wifi-office for today
        @param office_lunchbreak: array of begin- and end-time-stamp of lunch-break (if any)
        @param battery_percentage_when_booting: battery level of previous boot (only set if no charge event was in-between)
        """

        assert e_time.__class__ == datetime.datetime
        assert e_name.__class__ == unicode
        assert e_batt.__class__ == unicode
        assert e_uptime.__class__ == unicode
        assert (e_last_opposite_occurrence.__class__ == datetime.datetime or not e_last_opposite_occurrence)
        assert (e_last_occurrence.__class__ == datetime.datetime or not e_last_occurrence)
        assert (not battery_percentage_when_booting or battery_percentage_when_booting.__class__ == int)

        last_info = u''
        in_between_hms = u''
        in_between_s = u''
        ignore_occurrence = False

        # convert parameters to be writable:
        office_sum = prev_office_sum
        office_first_begin = prev_office_first_begin

        if e_last_opposite_occurrence:

            in_between_s = (e_time - e_last_opposite_occurrence).seconds + \
                (e_time - e_last_opposite_occurrence).days * 3600 * 24
            in_between_hms = unicode(OrgFormat.get_hms_from_sec(in_between_s))

            if e_name == u'boot':
                last_info = u' (off for '
            elif e_name == u'shutdown':
                last_info = u' (on for '
            elif e_name.endswith(u'-end'):
                last_info = u' (' + e_name[0:-4].replace('wifi-', '') + u' for '
            else:
                last_info = u' (not ' + e_name.replace('wifi-', '') + u' for '

            # handle special case: office hours
            additional_paren_string = ""
            if e_name == 'wifi-office-end':
                office_total = None
                # calculate office_sum and office_total
                if not office_sum:
                    office_sum = (e_time - e_last_opposite_occurrence).seconds
                    office_total = office_sum
                else:
                    assert(office_first_begin)
                    assert(office_sum)
                    office_sum = office_sum + (e_time - e_last_opposite_occurrence).seconds
                    office_total = int(time.mktime(e_time.timetuple()) - time.mktime(office_first_begin.timetuple()))

                assert(type(office_total) == int)
                assert(type(office_sum) == int)
                assert(type(in_between_s) == int)

                # come up with the additional office-hours string:
                additional_paren_string = u'; today ' + OrgFormat.get_hms_from_sec(office_sum) + \
                    '; today total ' + OrgFormat.get_hms_from_sec(office_total)

            if additional_paren_string:
                last_info += unicode(OrgFormat.get_dhms_from_sec(in_between_s)) + additional_paren_string + u')'
            else:
                last_info += unicode(OrgFormat.get_dhms_from_sec(in_between_s)) + u')'

        elif e_last_occurrence:

            in_between_s = (e_time - e_last_occurrence).seconds + \
                (e_time - e_last_occurrence).days * 3600 * 24
            in_between_hms = unicode(OrgFormat.get_hms_from_sec(in_between_s))

        # handle special case: office hours
        if e_name == 'wifi-office':
            if not office_sum or not office_first_begin:
                # new day
                office_first_begin = e_time
            else:
                # check if we've found a lunch-break (first wifi-office between 11:30-13:00 where not office for > 17min)
                if e_time.time() > datetime.time(11, 30) and e_time.time() < datetime.time(13, 00) and e_last_opposite_occurrence:
                    if e_last_opposite_occurrence.date() == e_time.date() and in_between_s > (17 * 60) and in_between_s < (80 * 60):
                        office_lunchbreak = [e_last_opposite_occurrence.time(), e_time.time()]

        # handle special case: boot without previous shutdown = crash
        if (e_name == u'boot') and \
                (e_last_occurrence and e_last_opposite_occurrence) and \
                (e_last_occurrence > e_last_opposite_occurrence):
            # last boot is more recent than last shutdown -> crash has happened
            last_info = u' after crash'
            in_between_hms = u''
            in_between_s = u''
            ignore_occurrence = True

        properties = OrgProperties()
        properties.add("IN-BETWEEN", in_between_hms)
        properties.add("IN-BETWEEN-S", unicode(in_between_s))
        properties.add("BATT-LEVEL", e_batt)
        properties.add("UPTIME", OrgFormat.get_hms_from_sec(int(e_uptime)))
        properties.add("UPTIME-S", e_uptime)

        if e_name == 'wifi-office-end' and office_lunchbreak:
            properties.add("OFFICE-SUMMARY",
                           e_last_opposite_occurrence.strftime('| %Y-%m-%d | %a ') +
                           prev_office_first_begin.strftime('| %H:%M ') +
                           office_lunchbreak[0].strftime('| %H:%M ') +
                           office_lunchbreak[1].strftime('| %H:%M ') +
                           e_time.strftime('| %H:%M | | |'))
        elif e_name == 'wifi-office-end' and not office_lunchbreak:
            properties.add("OFFICE-SUMMARY",
                           e_last_opposite_occurrence.strftime('| %Y-%m-%d | %a ') +
                           prev_office_first_begin.strftime('| %H:%M | 11:30 | 12:00 ') +
                           e_time.strftime('| %H:%M | | |'))
        elif e_name == 'shutdown':
            if battery_percentage_when_booting:
                batt_diff_from_boot_to_shutdown =  battery_percentage_when_booting - int(e_batt)
                if batt_diff_from_boot_to_shutdown >= 20:
                    # hypothetical run-time (in hours; derived from boot to shutdown) of the device for 100% battery capacity
                    # Note: battery_percentage_when_booting is set to False when a "charge-start"-event is recognized between boot and shutdown
                    # Note: only calculated when at least 20 percent difference of battery level between boot and shutdown
                    runtime_extrapolation = 100 * int(e_uptime) / batt_diff_from_boot_to_shutdown / 3600
                    properties.add("HOURS_RUNTIME_EXTRAPOLATION", runtime_extrapolation)

        self._writer.write_org_subitem(timestamp=e_time.strftime('<%Y-%m-%d %a %H:%M>'),
                                       output=e_name + last_info,
                                       properties=properties)

        return u'** ' + e_time.strftime('<%Y-%m-%d %a %H:%M>') + u' ' + e_name + last_info + \
            u'\n:PROPERTIES:\n:IN-BETWEEN: ' + in_between_hms + \
            u'\n:IN-BETWEEN-S: ' + unicode(in_between_s) + \
            u'\n:BATT-LEVEL: ' + e_batt + \
            u'\n:UPTIME: ' + unicode(OrgFormat.get_hms_from_sec(int(e_uptime))) + \
            u'\n:UPTIME-S: ' + unicode(e_uptime) + u'\n:END:\n', \
            ignore_occurrence, office_sum, office_first_begin, office_lunchbreak

    def _determine_opposite_eventname(self, e_name):
        """
        Takes a look at the event and returns the name of the opposite event description.
        Opposite of 'boot' is 'shutdown' (and vice versa).
        Opposite of 'foo' is 'foo-end' (and vice versa).

        @param e_name: string of an event name/description
        """

        assert (e_name.__class__ == unicode)

        if e_name == u'boot':
            return u'shutdown'
        elif e_name == u'shutdown':
            return u'boot'
        elif e_name.endswith(u'-end'):
            return e_name[0:-4]
        else:
            return e_name + u'-end'

    def _parse_data(self, data):
        """parses the phone log data"""

        last_occurrences = {}      # holds the previous occurrences of each event
        office_day = None          # holds the current day (in order to recognize day change)
        office_first_begin = None  # holds the time-stamp of the first appearance of wifi-office
        office_sum = None          # holds the sum of periods of all office-durations for this day
        office_lunchbreak = []     # array of begin and end time of lunch break
        battery_percentage_when_booting = False  # percentage of battery status of previous boot (only set if no charging event happened)

        for line in data.split('\n'):

            if not line:
                continue

            logging.debug("line: %s", line)

            components = re.match(self.LOGFILEENTRY_REGEX, line)

            if components:
                logging.debug("line matches")
            else:
                logging.debug("line does not match! (skipping this line)")
                continue

            # extracting the components to easy to use variables:
            datestamp = components.groups()[self.RE_ID_DATESTAMP].strip()
            hours = int(components.groups()[self.RE_ID_HOURS].strip())
            minutes = int(components.groups()[self.RE_ID_MINUTES].strip())
            e_name = unicode(components.groups()[self.RE_ID_NAME].strip())
            opposite_e_name = self._determine_opposite_eventname(e_name)
            e_batt = components.groups()[self.RE_ID_BATT].strip()
            e_uptime = components.groups()[self.RE_ID_UPTIME].strip()

            # generating a datestamp object from the time information:
            e_time = datetime.datetime(int(datestamp.split('-')[0]),
                                       int(datestamp.split('-')[1]),
                                       int(datestamp.split('-')[2]),
                                       hours, minutes)

            if e_name == 'boot':
                battery_percentage_when_booting = int(e_batt)
            elif e_name == 'charging-start':
                # set to False when a charging event is detected between boot and shutdown (which would render H_RUNTIME_EXTRAPOLATION useless)
                battery_percentage_when_booting = False
            elif e_name == 'shutdown' and opposite_e_name not in last_occurrences:
                # set to False when there is no boot in-between two shutdown events
                battery_percentage_when_booting = False

            # resetting office_day
            if e_name == 'wifi-office':
                if not office_day:
                    office_sum = None
                    office_day = datestamp
                    office_lunchbreak = []
                elif office_day != datestamp:
                    office_sum = None
                    office_day = datestamp
                    office_lunchbreak = []

            if opposite_e_name in last_occurrences:
                e_last_opposite_occurrence = last_occurrences[opposite_e_name]
            else:
                # no previous occurrence of the opposite event type
                e_last_opposite_occurrence = False

            if e_name in last_occurrences:
                last_time = last_occurrences[e_name]
            else:
                last_time = False

            result, ignore_occurrence, office_sum, office_first_begin, office_lunchbreak = \
                self._generateOrgentry(e_time, e_name, e_batt,
                                       e_uptime,
                                       e_last_opposite_occurrence,
                                       last_time,
                                       office_sum, office_first_begin, office_lunchbreak,
                                       battery_percentage_when_booting)

            ## update last_occurrences-dict
            if not ignore_occurrence:
                last_occurrences[e_name] = e_time

    def _main(self):
        """
        gets called automatically from Memacs class.
        read the lines from phonecalls backup xml file,
        parse and write them to org file
        """

        self._parse_data(CommonReader.get_data_from_file(self._args.phonelogfile))


# Local Variables:
# mode: flyspell
# eval: (ispell-change-dictionary "en_US")
# End:



# -*- coding: utf-8 -*-
# Time-stamp: <2014-05-03 17:46:44 vk>

import unittest
import os
from memacs.photos import PhotosMemacs


class TestPhotoMemacs(unittest.TestCase):

    def test_from_file(self):
        test_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data'
        )
        argv = "-s -f " + test_path
        memacs = PhotosMemacs(argv=argv.split())
        data = memacs.test_get_entries()

        filename = 'fujifilm-finepix40i.jpg'
        path = os.path.join(test_path, filename)

        self.assertEqual(
            data[0],
            "** <2000-08-04 Fri 18:22> [[%s][%s]]" % (path, filename)
        )
        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(
            data[2],
            u"   :ID:         c2833ac1c683dea5b600ac4f303a572d2148e1e7"
        )
        self.assertEqual(data[3], "   :END:")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

import logging
import time
import json
import sys
import os

import gpxpy
import gpxpy.gpx
import geocoder

from lib.orgproperty import OrgProperties
from lib.orgformat import OrgFormat
from lib.memacs import Memacs


class GPX(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--folder", dest="source",
            action="store", required=True,
            help="path to gpx file or folder")

        self._parser.add_argument(
            "-p", "--provider", dest="provider",
            action="store", default="google",
            help="geocode provider, default google")

        self._parser.add_argument(
            "-u", "--url", dest="url",
            action="store", help="url to nominatim server (osm only)")

        self._parser.add_argument(
            "--output-format", dest="output_format",
            action="store", default="{address}",
            help="format string to use for the headline")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

        if not os.path.exists(self._args.source):
            self._parser.error("source file or folder does not exist")

        if self._args.url and not self._args.url.startswith("http"):
            self._parser.error("invalid url given")

    def reverse_geocode(self, lat, lng):
        """get address for latitude/longitude"""

        if 'google' in self._args.provider:
            geocode = geocoder.google([lat, lng], method='reverse')

        elif 'osm' in self._args.provider:
            if not self._args.url:
                geocode = geocoder.osm([lat, lng], method='reverse')
                time.sleep(1)  # Nominatim Usage Policy
            else:
                if 'localhost' in self._args.url:
                    geocode = geocoder.osm([lat, lng], method='reverse', url='http://localhost/nominatim/search')
                else:
                    geocode = geocoder.osm([lat, lng], method='reverse', url=self._args.url)

        else:
            self._parser.error("invalid provider given")
            sys.exit(1)

        if not geocode.ok:
            logging.error("geocoding failed or api limit exceeded")
            sys.exit(1)
        else:
            logging.debug(geocode.json)
            return geocode.json

    def write_point(self, p):
        """write a point (including geocoding)"""

        timestamp = OrgFormat.datetime(p.time)
        geocode = self.reverse_geocode(p.latitude, p.longitude)
        output = self._args.output_format.decode('utf-8').format(**geocode)

        tags = []

        properties = OrgProperties(data_for_hashing=timestamp)

        if p.latitude:
            properties.add('LATITUDE', p.latitude)

        if p.longitude:
            properties.add('LONGITUDE', p.longitude)

        if p.source:
            tags.append(p.source.lower())

        if timestamp:
            self._writer.write_org_subitem(timestamp=timestamp,
                                           output=output,
                                           properties=properties,
                                           tags=tags)

    def handle_file(self, f):
        """iterate through a file"""

        data = open(f)
        gpx = gpxpy.parse(data)

        for track in gpx.tracks:
            for segment in track.segments:
                for point in segment.points:
                    self.write_point(point)
                    logging.debug(point)

    def _main(self):
        """
        get's automatically called from Memacs class
        """

        if os.path.isfile(self._args.source):
            self.handle_file(self._args.source)

        else:
            for root, dirs, files in os.walk(self._args.source):
                for f in files:
                    if f.endswith('.gpx'):
                        self.handle_file(os.path.join(root, f))



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2015-04-30 17:12:02 vs>

from memacs.mu import MuMail

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2015-03-08"
PROG_SHORT_DESCRIPTION = u"Memacs for Mu Mails"
PROG_TAG = u"emails:mumail"
PROG_DESCRIPTION = u"""This memacs module will connect mu mail database,
fetch all mails and writes them to an orgfile.
"""
CONFIG_PARSER_NAME = ""
COPYRIGHT_YEAR = "2011-2015"
COPYRIGHT_AUTHORS = """Karl Voit <tools@Karl-Voit.at>,
Stephanus Volke <post@stephanus-volke.de>"""

if __name__ == "__main__":
    
    memacs = MuMail(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_description=PROG_DESCRIPTION,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS,
        use_config_parser_name=CONFIG_PARSER_NAME
        )
    memacs.handle_main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2011-12-30 03:38:09 armin>

import argparse
import logging
import time
import sys
import json
import datetime

from lib.orgformat import OrgFormat
from lib.memacs import Memacs
from lib.reader import UnicodeDictReader
from lib.orgproperty import OrgProperties


class Csv(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
           "-f", "--file", dest="csvfile", required=True,
           action="store", help="input csv file", type=argparse.FileType('rb'))

        self._parser.add_argument(
           "-d", "--delimiter", dest="delimiter", default=";",
           action="store", help='delimiter, default ";"')

        self._parser.add_argument(
           "-e", "--encoding", dest="encoding",
           action="store", default="utf-8", help="default encoding utf-8, " +
           "see http://docs.python.org/library/codecs.html#standard-encodings" +
           "for possible encodings")

        self._parser.add_argument(
            "-n", "--fieldnames", dest="fieldnames", default=None,
            action="store", help="header field names of the columns",
            type=str.lower)

        self._parser.add_argument(
            "-p", "--properties", dest="properties", default='',
            action="store", help="fields to use for properties",
            type=str.lower)

        self._parser.add_argument(
            "--timestamp-field", dest="timestamp_field", required=True,
            action="store", help="field name of the timestamp",
            type=str.lower)

        self._parser.add_argument(
            "--timestamp-format", dest="timestamp_format",
            action="store", help='format of the timestamp, i.e. ' +
            '"%d.%m.%Y %H:%M:%S" for "14.02.2012 10:22:37" ' +
            'see http://docs.python.org/library/time.html#time.strftime' +
            'for possible formats, default unix timestamp')

        self._parser.add_argument(
            "--output-format", dest="output_format", required=True,
            action="store", help="format string to use for the output")

        self._parser.add_argument(
            "--skip-header", dest="skip_header",
            action="store_true", help="skip first line of the csv file")

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

        if self._args.fieldnames:
            self._args.fieldnames = [name.strip() for name in self._args.fieldnames.split(',')]

    def _handle_row(self, row):
        """
        handle a single row
        """

        try:
            # assume unix timestamp
            if not self._args.timestamp_format:
                timestamp = datetime.datetime.fromtimestamp(int(row[self._args.timestamp_field]))
            else:
                timestamp = time.strptime(row[self._args.timestamp_field], self._args.timestamp_format)

            # show time with the timestamp format, but only
            # if it contains at least hours and minutes
            if not self._args.timestamp_format and \
            (x in self._args.timestamp_format for x in ['%H', '%M']):
                timestamp = OrgFormat.datetime(timestamp)
            else:
                timestamp = OrgFormat.date(timestamp)

        except ValueError, e:
            logging.error("timestamp-format does not match: %s", e)
            sys.exit(1)

        except IndexError, e:
            logging.error("did you specify the right delimiter?", e)
            sys.exit(1)

        properties = OrgProperties(data_for_hashing=json.dumps(row))
        output = self._args.output_format.format(**row)

        if self._args.properties:
            for prop in self._args.properties.split(','):
                properties.add(prop.upper().strip(), row[prop])

        self._writer.write_org_subitem(timestamp=timestamp,
                                       output=output,
                                       properties=properties)

    def _main(self):
        """
        get's automatically called from Memacs class
        """

        with self._args.csvfile as f:

            try:
                reader = UnicodeDictReader(f,
                                           self._args.delimiter,
                                           self._args.encoding,
                                           self._args.fieldnames)

                if self._args.skip_header:
                    next(reader)

                for row in reader:
                    self._handle_row(row)
                    logging.debug(row)

            except TypeError, e:
                logging.error("not enough fieldnames or wrong delimiter given")
                logging.debug("Error: %s" % e)
                sys.exit(1)

            except UnicodeDecodeError, e:
                logging.error("could not decode file in utf-8, please specify input encoding")
                sys.exit(1)



# -*- coding: utf-8 -*-
# Time-stamp: <2012-09-06 22:02:48 armin>

import os
import re
import time
import unittest
import xml.etree.ElementTree as ET

from memacs.phonecalls import PhonecallsMemacs


class TestPhonecallsMemacs(unittest.TestCase):

    def setUp(self):
        self._test_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'calls.xml'
        )
        argv = "-s -f " + self._test_file
        memacs = PhonecallsMemacs(argv=argv.split())
        self.data = memacs.test_get_entries()
        self._calls = ET.parse(self._test_file)

    def test_from_file(self):
        data = self.data

        for i in range(8):
            self._assertPhoneLog(i, data[i*7:(i+1)*7])

    def _assertPhoneLog(self, index, call_data):
        call = self._calls.findall('call')[index]

        duration = call_data[2].split()[-1]
        number = call_data[4].split()[-1]
        start, _ = re.findall('<(.*?)>', call_data[0])
        date = time.gmtime(int(call.get('date'))/1000.)

        self.assertEqual(time.strftime('%Y-%m-%d %a %H:%M', date), start)
        self.assertEqual(duration, call.get('duration'))
        self.assertEqual(number, call.get('number'))



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2013-09-14 14:49:06 vk>

import logging
import time
from datetime import datetime
from dateutil import parser
import os
import sys
from twython import Twython, TwythonError
from lib.orgformat import OrgFormat
from lib.memacs import Memacs
from lib.reader import UnicodeCsvReader
from lib.orgproperty import OrgProperties

class Twitter(Memacs):
    def _main(self):
        APP_KEY = self._get_config_option("APP_KEY")

        APP_SECRET = self._get_config_option("APP_SECRET")

        OAUTH_TOKEN = self._get_config_option("OAUTH_TOKEN")

        OAUTH_TOKEN_SECRET = self._get_config_option("OAUTH_TOKEN_SECRET")

        screen_name = self._get_config_option("screen_name")

        count = self._get_config_option("count")

        twitter = Twython(
            APP_KEY,
            APP_SECRET,
            OAUTH_TOKEN,
            OAUTH_TOKEN_SECRET
            )
        try:
            home_timeline = twitter.get_home_timeline(screenname=screen_name, count=count)

        except TwythonError as e:
            logging.error(e)
            sys.exit(1)

        for tweet in home_timeline:
            # strptime doesn't support timezone info, so we are using dateutils.
            date_object = parser.parse(tweet['created_at'])

            timestamp = OrgFormat.datetime(date_object)
            try:
                # Data is already Unicode, so don't try to re-encode it.
                output = tweet['text']
            except:
               logging.error(sys.exc_info()[0])
               print "Error: ", sys.exc_info()[0]

            data_for_hashing = output + timestamp + output
            properties = OrgProperties(data_for_hashing=data_for_hashing)

            properties.add("name", tweet['user']['name'])
            properties.add("twitter_id", tweet['id'])
            properties.add("contributors", tweet['contributors'])
            properties.add("truncated", tweet['truncated'])
            properties.add("in_reply_to_status_id", tweet['in_reply_to_status_id'])
            properties.add("favorite_count", tweet['favorite_count'])
            properties.add("source", tweet['source'])
            properties.add("retweeted", tweet['retweeted'])
            properties.add("coordinates", tweet['coordinates'])
            properties.add("entities", tweet['entities'])

            self._writer.write_org_subitem(timestamp=timestamp,
                                          output = output,
                                          properties = properties)



# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31 aw>

import unittest
import os
from memacs.csv import Csv


class TestCsv(unittest.TestCase):

    def test_example1(self):
        example1 = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'example1.csv'
        )

        argv = []
        argv.append("-f")
        argv.append(example1)
        argv.append("--fieldnames")
        argv.append("date,text,value,currency,")
        argv.append("--timestamp-field")
        argv.append("date")
        argv.append("--timestamp-format")
        argv.append("%d.%m.%Y")
        argv.append("--output-format")
        argv.append("{text}")
        argv.append("--properties")
        argv.append("currency,value")

        memacs = Csv(argv=argv)
        data = memacs.test_get_entries()

        self.assertEqual(data[0], "** <2012-02-23 Thu> Amazon")
        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(data[2], "   :CURRENCY:   EUR")
        self.assertEqual(data[3], "   :VALUE:      100,00")
        self.assertEqual(data[4], "   :ID:         3f4898135bc340ede51aff4519ebd54db92fe23c")
        self.assertEqual(data[5], "   :END:")

    def test_example2(self):
        example1 = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'example2.csv'
        )

        argv = []
        argv.append("--delimiter")
        argv.append("|")
        argv.append("-f")
        argv.append(example1)
        argv.append("--timestamp-field")
        argv.append("date")
        argv.append("--output-format")
        argv.append("{text}")

        memacs = Csv(argv=argv)
        data = memacs.test_get_entries()

        self.assertEqual(data[0], "** <2012-02-23 Thu 14:40> Alibaba")
        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(data[2], "   :ID:         08cfc2bf06e9f2a235641912ea7c7d7c87072ad3")
        self.assertEqual(data[3], "   :END:")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from memacs.gpx import GPX

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2017-03-02"
PROG_SHORT_DESCRIPTION = u"Memacs for GPX files"
PROG_TAG = u"gps"

COPYRIGHT_YEAR = "2017"
COPYRIGHT_AUTHORS = """Manuel Koell <mankoell@gmail.com>"""


if __name__ == "__main__":
    memacs = GPX(prog_version=PROG_VERSION_NUMBER,
                 prog_version_date=PROG_VERSION_DATE,
                 prog_short_description=PROG_SHORT_DESCRIPTION,
                 prog_tag=PROG_TAG,
                 copyright_year=COPYRIGHT_YEAR,
                 copyright_authors=COPYRIGHT_AUTHORS)
    memacs.handle_main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2011-12-19 15:13:31 aw>

import os
import shutil
import tempfile
import unittest

from memacs.filenametimestamps import FileNameTimeStamps


class TestFileNameTimeStamps(unittest.TestCase):

    def setUp(self):
        self._tmp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self._tmp_dir)

    def test_functional(self):
        # touch file
        tmpfile = os.path.join(self._tmp_dir, '2011-12-19T23.59.12_test1.txt')
        with open(tmpfile, 'w'):
            pass

        entry = "** <2011-12-19 Mon 23:59> [[" + tmpfile + \
            "][2011-12-19T23.59.12_test1.txt]]"

        argv = "-s -f " + self._tmp_dir
        memacs = FileNameTimeStamps(argv=argv.split())
        data = memacs.test_get_entries()

        self.assertEqual(data[0], entry)
        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(data[3], "   :END:")



#!/usr/bin/python2
# -*- coding: utf-8 -*-

import argparse
import datetime
import logging
import sys

import batinfo

from lib.orgproperty import OrgProperties
from lib.orgformat import OrgFormat
from lib.memacs import Memacs

ROOT = '/sys/class/power_supply'


class Battery(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-b", "--battery", dest="name",
            action="store", default="BAT0",
            help="select battery to read stats from")

        self._parser.add_argument(
            "-p", "--path", dest="path",
            action="store", default=ROOT,
            help=argparse.SUPPRESS)

        self._parser.add_argument(
            "--output-format", dest="output_format",
            action="store", default="{battery.name}",
            help="format string to use for the output"
        )

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

    def _handle_battery(self, bat):
        """
        handle single battery, e.g. BAT0
        """

        # calculate watt usage
        consumption = float(bat.current_now / 1000000.0 *
                            bat.voltage_now / 1000000.0)

        timestamp = OrgFormat.datetime(datetime.datetime.now())
        output = self._args.output_format.format(battery=bat)

        properties = OrgProperties(data_for_hashing=timestamp)
        properties.add("CYCLE_COUNT", bat.cycle_count)
        properties.add("CAPACITY", '%s%%' % bat.capacity)
        properties.add("STATUS", bat.status.lower())

        if consumption:
            properties.add("CONSUMPTION", '%.1f W' % consumption)

        self._writer.write_org_subitem(timestamp=timestamp,
                                       output=output,
                                       properties=properties)

    def _main(self):
        """
        get's automatically called from Memacs class
        """

        try:
            batteries = batinfo.Batteries(self._args.path)

            for bat in batteries.stat:
                if self._args.name in bat.name:
                    self._handle_battery(bat)

        except OSError, e:
            logging.error("no battery present")
            sys.exit(1)


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2015-01-30 18:30:03 vk>

import sys
import os
import logging
import xml.sax
import time
import re          ## RegEx for detecting patterns
import codecs      ## Unicode conversion
import HTMLParser  ## un-escaping HTML entities like emojis
import tempfile    ## create temporary files
from xml.sax._exceptions import SAXParseException
from lib.orgformat import OrgFormat
from lib.orgproperty import OrgProperties
from lib.memacs import Memacs
from lib.reader import CommonReader


class SmsSaxHandler(xml.sax.handler.ContentHandler):
    """
    Sax handler for sms backup xml files.
    See documentation memacs_sms.org for an example.
    """

    ## from https://github.com/wooorm/emoji-emotion/blob/master/Support.md
    EMOJIS = {
        u'\ud83d\udc7f':'imp',
        u'\ud83d\ude3e':'pouting_cat',
        u'\ud83d\ude21':'rage',
        u'\ud83d\ude20':'angry',
        u'\ud83d\ude27':'anguished',
        u'\ud83d\ude2d':'sob',
        u'\ud83d\ude31':'scream',
        u'\ud83d\ude40':'scream_cat',
        u'\ud83d\ude08':'smiling_imp',
        u'\ud83d\ude1f':'worried',
        u'\ud83d\ude3f':'crying_cat_face',
        u'\ud83d\ude15':'confused',
        u'\ud83d\ude16':'confounded',
        u'\ud83d\ude30':'cold_sweat',
        u'\ud83d\ude22':'cry',
        u'\ud83d\ude1e':'disappointed',
        u'\ud83d\ude33':'flushed',
        u'\ud83d\ude28':'fearful',
        u'\ud83d\ude2c':'grimacing',
        u'\ud83d\ude2e':'open_mouth',
        u'\ud83d\ude23':'persevere',
        u'\ud83d\ude2b':'tired_face',
        u'\ud83d\ude12':'unamused',
        u'\ud83d\ude29':'weary',
        u'\ud83d\ude35':'dizzy_face',
        u'\ud83d\ude25':'disappointed_relieved',
        u'\ud83d\ude26':'frowning',
        u'\ud83d\ude01':'grin',
        u'\ud83d\ude2f':'hushed',
        u'\ud83d\ude37':'mask',
        u'\ud83d\ude14':'pensive',
        u'\ud83d\ude13':'sweat',
        u'\ud83d\ude1c':'stuck_out_tongue_winking_eye',
        u'\ud83d\ude11':'expressionless',
        u'\ud83d\ude36':'no_mouth',
        u'\ud83d\ude10':'neutral_face',
        u'\ud83d\ude34':'sleeping',
        u'\ud83d\ude1d':'stuck_out_tongue_closed_eyes',
        u'\ud83d\ude2a':'sleepy',
        u'\ud83d\ude06':'laughing; satisfied',
        u'\ud83d\ude0e':'sunglasses',
        u'\ud83d\ude1b':'stuck_out_tongue',
        u'\ud83d\ude32':'astonished',
        u'\ud83d\ude0a':'blush',
        u'\ud83d\ude00':'grinning',
        u'\ud83d\ude3d':'kissing_cat',
        u'\ud83d\ude19':'kissing_smiling_eyes',
        u'\ud83d\ude17':'kissing',
        u'\ud83d\ude1a':'kissing_closed_eyes',
        u'\u263a\ufe0f':'relaxed',
        u'\ud83d\ude0c':'relieved',
        u'\ud83d\ude04':'smile',
        u'\ud83d\ude3c':'smirk_cat',
        u'\ud83d\ude38':'smile_cat',
        u'\ud83d\ude03':'smiley',
        u'\ud83d\ude3a':'smiley_cat',
        u'\ud83d\ude05':'sweat_smile',
        u'\ud83d\ude0f':'smirk',
        u'\ud83d\ude3b':'heart_eyes_cat',
        u'\ud83d\ude0d':'heart_eyes',
        u'\ud83d\ude07':'innocent',
        u'\ud83d\ude02':'joy',
        u'\ud83d\ude39':'joy_cat',
        u'\ud83d\ude18':'kissing_heart',
        u'\ud83d\ude09':'wink',
        u'\ud83d\ude0b':'yum',
        u'\ud83d\ude24':'triumph'}

    EMOJI_ENCLOSING_CHARACTER = "~" ## character which encloses emojis found ~wink~


    def __init__(self, writer, ignore_incoming, ignore_outgoing, numberdict):
        """
        Ctor

        @param writer: orgwriter
        @param ignore_incoming: ignore incoming smses
        """
        self._writer = writer
        self._ignore_incoming = ignore_incoming
        self._ignore_outgoing = ignore_outgoing
        self._numberdict = numberdict


    def startElement(self, name, attrs):
        """
        at every <sms> tag write to orgfile
        """
        logging.debug("Handler @startElement name=%s,attrs=%s", name, attrs)
        htmlparser = HTMLParser.HTMLParser()

        if name == "sms":
            sms_subject = attrs['subject']
            sms_date = int(attrs['date']) / 1000     # unix epoch
            sms_body = attrs['body']
            sms_address = attrs['address'].strip().replace('-',u'').replace('/',u'').replace(' ',u'').replace('+',u'00')
            sms_type_incoming = int(attrs['type']) == 1
            contact_name = False
            if 'contact_name' in attrs:
                ## NOTE: older version of backup app did not insert contact_name into XML
                contact_name = attrs['contact_name']
            else:
                if self._numberdict:
                    if sms_address in self._numberdict.keys():
                        contact_name = self._numberdict[sms_address]

            skip = False

            if sms_type_incoming == True:
                output = "SMS from "
                if self._ignore_incoming:
                    skip = True
            else:
                output = "SMS to "
                if self._ignore_outgoing:
                    skip = True

            if not skip:

                name_string = ""
                if contact_name:
                    name_string = '[[contact:' + contact_name + '][' + contact_name + ']]'
                else:
                    name_string = "Unknown"
                output += name_string + ": "

                ## reverse encoding hack from just before:
                sms_body = htmlparser.unescape(sms_body.replace(u'EnCoDiNgHaCk42', u'&#'))
                for emoji in self.EMOJIS.keys():
                    ## FIXXME: this is a horrible dumb brute-force algorithm.
                    ##         In case of bad performance, this can be optimized dramtically
                    sms_body = sms_body.replace(emoji, self.EMOJI_ENCLOSING_CHARACTER + \
                                                self.EMOJIS[emoji] + self.EMOJI_ENCLOSING_CHARACTER).replace(u'\n', u'⏎')

                if sms_subject != "null":
                    # in case of MMS we have a subject
                    output += sms_subject
                    notes = sms_body
                else:
                    output += sms_body
                    notes = ""

                timestamp = OrgFormat.datetime(time.gmtime(sms_date))
                data_for_hashing = output + timestamp + notes
                properties = OrgProperties(data_for_hashing=data_for_hashing)

                properties.add("NUMBER", sms_address)
                properties.add("NAME", contact_name)

                self._writer.write_org_subitem(output=output,
                                               timestamp=timestamp,
                                               note=notes,
                                               properties=properties)


class SmsMemacs(Memacs):

    _numberdict = False

    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--file", dest="smsxmlfile",
            action="store", required=True,
            help="path to sms xml backup file")

        self._parser.add_argument(
            "--ignore-incoming", dest="ignore_incoming",
            action="store_true",
            help="ignore incoming smses")

        self._parser.add_argument(
            "--ignore-outgoing", dest="ignore_outgoing",
            action="store_true",
            help="ignore outgoing smses")

        self._parser.add_argument(
            "--orgcontactsfile", dest="orgcontactsfile",
            action="store", required=False,
            help="path to Org-contacts file for phone number lookup. Phone numbers have to match.")


    def parse_org_contact_file(self, orgfile):
        """
        Parses the given Org-mode file for contact entries.

        The return format is a follows:
        numbers = {'004369912345678':'First2 Last1', '0316987654':'First2 Last2', ...}

        @param orgfile: file name of a Org-mode file to parse
        @param return: list of dict-entries containing the numbers to name dict
        """

        linenr = 0

        ## defining distinct parsing status states:
        headersearch = 21
        propertysearch = 42
        inproperty = 73
        status = headersearch

        contacts = {}
        current_name = u''

        HEADER_REGEX = re.compile('^(\*+)\s+(.*?)(\s+(:\S+:)+)?$')
        PHONE = '\s+([\+\d\-/ ]{7,})$'
        PHONE_REGEX = re.compile(':(PHONE|oldPHONE|MOBILE|oldMOBILE|HOMEPHONE|oldHOMEPHONE|WORKPHONE|oldWORKPHONE):' + PHONE)

        for rawline in codecs.open(orgfile, 'r', encoding='utf-8'):
            line = rawline.strip()   ## trailing and leading spaces are stupid
            linenr += 1

            header_components = re.match(HEADER_REGEX, line)
            if header_components:
                ## in case of new header, make new currententry because previous one was not a contact header with a property
                current_name = header_components.group(2)
                status = propertysearch
                continue

            if status == headersearch:
                ## if there is something to do, it was done above when a new heading is found
                continue

            if status == propertysearch:
                if line == u':PROPERTIES:':
                    status = inproperty
                continue

            elif status == inproperty:

                phone_components = re.match(PHONE_REGEX, line)
                if phone_components:
                    phonenumber = phone_components.group(2).strip().replace('-',u'').replace('/',u'').replace(' ',u'').replace('+',u'00')
                    contacts[phonenumber] = current_name
                elif line == u':END:':
                    status = headersearch

                continue

            else:
                ## I must have mixed up status numbers or similar - should never be reached.
                logging.error("Oops. Internal parser error: status \"%s\" unknown. The programmer is an idiot. Current contact entry might get lost due to recovering from that shock. (line number %s)" % (str(status), str(linenr)))
                status = headersearch
                continue

        logging.info("found %s suitable contacts while parsing \"%s\"" % (str(len(contacts)), orgfile))
        return contacts


    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)
        if not (os.path.exists(self._args.smsxmlfile) or \
                     os.access(self._args.smsxmlfile, os.R_OK)):
            self._parser.error("input file not found or not readable")

        if self._args.orgcontactsfile:
            if not (os.path.exists(self._args.orgcontactsfile) or \
                    os.access(self._args.orgcontactsfile, os.R_OK)):
                self._parser.error("Org-contacts file not found or not readable")
            self._numberdict = self.parse_org_contact_file(self._args.orgcontactsfile)


    def _main(self):
        """
        get's automatically called from Memacs class
        read the lines from sms backup xml file,
        parse and write them to org file
        """

        ## replace HTML entities "&#" in original file to prevent XML parser from worrying:
        temp_xml_file = tempfile.mkstemp()[1]
        line_number = 0
        logging.debug("tempfile [%s]", str(temp_xml_file))
        with codecs.open(temp_xml_file, 'w', encoding='utf-8') as outputhandle:
            for line in codecs.open(self._args.smsxmlfile, 'r', encoding='utf-8'):
                try:
                    ## NOTE: this is a dirty hack to prevent te XML parser from complainaing about
                    ##       encoding issues of UTF-8 encoded emojis. Will be reverted when parsing sms_body
                    outputhandle.write(line.replace(u'&#', u'EnCoDiNgHaCk42') + u'\n')
                except IOError as e:
                    print "tempfile line " + str(line_number) +  " [" + str(temp_xml_file) + "]"
                    print "I/O error({0}): {1}".format(e.errno, e.strerror)
                except ValueError as e:
                    print "tempfile line " + str(line_number) +  " [" + str(temp_xml_file) + "]"
                    print "Value error: {0}".format(e)
                    #print "line [%s]" % str(line)
                except:
                    print "tempfile line " + str(line_number) +  " [" + str(temp_xml_file) + "]"
                    print "Unexpected error:", sys.exc_info()[0]
                    raise

        data = CommonReader.get_data_from_file(temp_xml_file)

        try:
            xml.sax.parseString(data.encode('utf-8'),
                                SmsSaxHandler(self._writer,
                                              self._args.ignore_incoming,
                                              self._args.ignore_outgoing,
                                              self._numberdict))
        except SAXParseException:
            logging.error("No correct XML given")
            sys.exit(1)
        else:
            os.remove(temp_xml_file)



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2013-04-04 16:18:07 vk>

from memacs.twitter import Twitter

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2013-09-01"
PROG_SHORT_DESCRIPTION = u"Memacs for Twitter "
PROG_TAG = u"mytag"
PROG_DESCRIPTION = u"""
This Memacs module will process your Twitter timeline ....


sample config:
[memacs-twitter]           <-- "memacs-example" has to be CONFIG_PARSER_NAME
APP_KEY =
APP_SECRET =
OAUTH_TOKEN =
OAUTH_TOKEN_SECRET =
screen_name =
count =


"""
# set CONFIG_PARSER_NAME only, when you want to have a config file
# otherwise you can comment it out
CONFIG_PARSER_NAME="memacs-twitter"
COPYRIGHT_YEAR = "2013"
COPYRIGHT_AUTHORS = """Ian Barton <ian@manor-farm.org>"""


if __name__ == "__main__":
    memacs = Twitter(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_description=PROG_DESCRIPTION,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS,
        use_config_parser_name=CONFIG_PARSER_NAME
        )
    memacs.handle_main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2017-02-07 19:25 manu>

import subprocess
import distutils.spawn
import os.path
import logging
import calendar
import time
import sys
import io

from lib.reader import UnicodeCsvReader
from lib.orgproperty import OrgProperties
from lib.memacs import Memacs

ARBTT_STATS = 'arbtt-stats'
ARBTT_FORMAT = '%m/%d/%y %H:%M:%S'


class Arbtt(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "--logfile",
            dest="log",
            action="store",
            help="use this file instead of ~/.arbtt/capture.log")

        self._parser.add_argument(
            "--categorizefile",
            dest="cfg",
            action="store",
            help="use this file instead of ~/.arbtt/categorize.cfg")

        self._parser.add_argument(
            "--also-inactive",
            dest="inactive",
            action="store_true",
            help="include inactive samples")

        self._parser.add_argument(
            "--intervals",
            dest="intervals",
            action="append", required=True,
            help="list intervals of tag or category " + \
                 "(the latter has to end with a colon)"
        )

        self._parser.add_argument(
            "--csv",
            dest="csv",
            action="store",
            help="csv file"
        )

    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)

        if self._args.log:
            if not os.path.isfile(self._args.log):
                self._parser.error("logfile does not exist")

        if self._args.cfg:
            if not os.path.isfile(self._args.cfg):
                self._parser.error("config file does not exist")

        if self._args.csv:
            if not os.path.isfile(self._args.csv):
                self._parser.error("csv file does not exist")

        if self._args.csv and self._args.log or self._args.inactive:
            self._parser.error("You gave both \"--csv\" and \"--logfile\" or \"--also-inactive\" argument." + \
                               "Please use either or.")

        if not self._args.intervals:
            self._parser.error("No intervals tag or category specified")

    def get_sec(self, t):
        """
        get H:M:S as seconds

        @param t: hms string
        @return: seconds
        """
        h, m, s = [int(i) for i in t.split(':')]
        return h*3600 + m*60 + s

    def get_timestamp(self, dt):
        """
        1. parse datetime string
        2. utc to local time

        @param dt: datetime string
        @return: datetime in org format
        """
        dt_tuple = time.strptime(dt, ARBTT_FORMAT)
        dt_local = time.localtime(calendar.timegm(dt_tuple))

        return time.strftime('<%Y-%m-%d %a %H:%M:%S>', dt_local)

    def get_timerange(self, begin, end):
        """
        return a date+time range (including seconds)

        @param begin: start date (string)
        @param end: end date (string)
        @return: datetime range in org format
        """
        return "%s--%s" % (self.get_timestamp(begin),
                           self.get_timestamp(end))

    def __parse_sample(self, target, row):
        """
        parse a row of csv and write entry

        @param target: tag or category
        @param row: list of columns
        """
        tag, begin, end, duration = row

        timestamp = self.get_timerange(begin, end)
        duration = self.get_sec(duration)

        properties = OrgProperties(data_for_hashing=timestamp)
        properties.add('DURATION', duration)

        tags = []

        # remove colon from output
        if target.endswith(':'):
            target = target[:-1]
            tags.append(target)
        elif ':' in target:
            target = target.split(':')[0]

        output = target.capitalize()
        tags.append(tag)

        self._writer.write_org_subitem(
            timestamp=timestamp,
            output=output,
            tags=tags,
            properties=properties
        )

    def __handle_intervals(self, target):
        """
        handles an interval of data records

        @param target: tag or category
        """
        command = [
            ARBTT_STATS,
            '--output-format=csv',
            '--intervals=%s' % target
        ]

        if self._args.log:
            command.append('--logfile=%s' % self._args.log)

        if self._args.cfg:
            command.append('--categorizefile=%s' % self._args.cfg)

        if self._args.inactive:
            command.append('--also-inactive')

        if not self._args.csv:
            stats = subprocess.check_output(command)
            f = io.StringIO(unicode(stats))
        else:
            # skip dump of stats data
            f = open(self._args.csv)

        # skip header columns
        reader = UnicodeCsvReader(f, delimiter=',')
        next(reader)

        for row in reader:
            self.__parse_sample(target, row)

    def _main(self):
        """
        get's automatically called from Memacs class
        """
        # check if arbtt is installed
        if distutils.spawn.find_executable(ARBTT_STATS) is None:
            logging.error(ARBTT_STATS + ': command not found')
            sys.exit(1)

        for target in self._args.intervals:
            self.__handle_intervals(target)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Time-stamp: <2017-02-07 19:25 manu>

from memacs.arbtt import Arbtt

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2017-02-07"
PROG_SHORT_DESCRIPTION = u"Memacs for arbtt"
PROG_TAG = u"arbtt"
PROG_DESCRIPTION = u"""
This Memacs module will parse arbtt stats ....

"""

COPYRIGHT_YEAR = "2017"
COPYRIGHT_AUTHORS = """Manuel Koell <mankoell@gmail.com>"""


if __name__ == "__main__":
    memacs = Arbtt(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_description=PROG_DESCRIPTION,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS
        )
    memacs.handle_main()



# -*- coding: utf-8 -*-

import unittest
import os

from memacs.gpx import GPX


class TestGPX(unittest.TestCase):

    def test_google(self):
        sample = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'sample.gpx'
        )

        argv = []
        argv.append('-f')
        argv.append(sample)
        
        memacs = GPX(argv=argv)
        data = memacs.test_get_entries()

        self.assertEqual(
            data[0],
            u"** <2017-04-01 Sat 10:50> Eggenberger Allee 9, 8020 Graz, Austria	:network:")
        self.assertEqual(
            data[1],
            "   :PROPERTIES:")
        self.assertEqual(
            data[2],
            "   :LATITUDE:   47.0693")
        self.assertEqual(
            data[3],
            "   :LONGITUDE:  15.4076001")
        self.assertEqual(
            data[4],
            "   :ID:         c2dc4f2289d79cff4cf27faa95863f8cb5b8cb21")
        self.assertEqual(
            data[5],
            "   :END:")

    def test_osm(self):
        sample = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'sample.gpx'
        )

        argv = []
        argv.append('-f')
        argv.append(sample)
        argv.append('-p osm')
        
        memacs = GPX(argv=argv)
        data = memacs.test_get_entries()

        self.assertEqual(
            data[0],
            u"** <2017-04-01 Sat 10:50> FH Joanneum - Prüffeld, 150, Alte Poststraße, Gries, Graz, Steiermark, 8020, Österreich	:network:")
        self.assertEqual(
            data[1],
            "   :PROPERTIES:")
        self.assertEqual(
            data[2],
            "   :LATITUDE:   47.0693")
        self.assertEqual(
            data[3],
            "   :LONGITUDE:  15.4076001")
        self.assertEqual(
            data[4],
            "   :ID:         c2dc4f2289d79cff4cf27faa95863f8cb5b8cb21")
        self.assertEqual(
            data[5],
            "   :END:")



# -*- coding: utf-8 -*-
# Time-stamp: <2011-10-28 15:13:31>

import os
import time
import unittest
from memacs.arbtt import Arbtt


class TestArbtt(unittest.TestCase):

    def setUp(self):
        self.memacs = Arbtt()

        self.sample = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data', 'sample-arbtt.csv'
        )

    def test_get_sec(self):
        self.assertEqual(self.memacs.get_sec('00:15:00'), 900)

    def test_get_timestamp(self):
        # assuming utc+1 w/o dst :)
        self.assertEqual(
            self.memacs.get_timestamp('02/05/17 19:00:00'),
            '<2017-02-05 Sun 20:00:00>'
        )

    def test_get_timerange(self):
        # assuming utc+1 w/o dst :)
        self.assertEqual(
            self.memacs.get_timerange('02/05/17 19:00:00', '02/05/17 19:15:00'),
            '<2017-02-05 Sun 20:00:00>--<2017-02-05 Sun 20:15:00>'
        )

    def test_functional(self):

        argv = []
        argv.append('-s')
        argv.append('--intervals=web')
        # argv.append('--logfile=%s' % self.log)
        # argv.append('--categorizefile=%s' % self.cfg)
        argv.append('--csv=%s' % self.sample)

        memacs = Arbtt(argv=argv)
        data = memacs.test_get_entries()

        entry = '** <2017-02-05 Sun 20:00:00>--<2017-02-05 Sun 20:15:00> Web\t:web:'

        self.assertEqual(data[0], entry)
        self.assertEqual(data[1], "   :PROPERTIES:")
        self.assertEqual(data[2], "   :DURATION:   900")
        self.assertEqual(data[4], "   :END:")




#!/usr/bin/python2
# -*- coding: utf-8 -*-

from memacs.lastfm import LastFM


CONFIG_PARSER_NAME="memacs-lastfm"

PROG_VERSION_NUMBER = u"0.1"
PROG_VERSION_DATE = u"2017-02-24"
PROG_SHORT_DESCRIPTION = u"Memacs for lastfm"
PROG_TAG = u"lastfm"

COPYRIGHT_YEAR = "2017"
COPYRIGHT_AUTHORS = """Manuel Koell <mankoell@gmail.com>"""

if __name__ == "__main__":
    memacs = LastFM(
        prog_version=PROG_VERSION_NUMBER,
        prog_version_date=PROG_VERSION_DATE,
        prog_short_description=PROG_SHORT_DESCRIPTION,
        prog_tag=PROG_TAG,
        copyright_year=COPYRIGHT_YEAR,
        copyright_authors=COPYRIGHT_AUTHORS,
        use_config_parser_name=CONFIG_PARSER_NAME
    )
    memacs.handle_main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# Time-stamp: <2013-05-02 20:36:03 vk>

import sys
import os
import logging
import xml.sax
import time, datetime
from xml.sax._exceptions import SAXParseException
from lib.orgformat import OrgFormat
from lib.memacs import Memacs
from lib.reader import CommonReader
from lib.orgproperty import OrgProperties
#import pdb

logging.basicConfig(filename='debug.log', level=logging.DEBUG)

class PhonecallsSaxHandler(xml.sax.handler.ContentHandler):
    """
    Sax handler for following xml's:

    <?xml version="1.0" encoding="UTF-8"?>
    <alllogs count="500">
            <log number="01270811333" time="3 Sep 2013 10:03:26" date="1378199006383" type="1" name="" new="1" dur="30" />
            <log number="01270588896" time="1 Sep 2013 19:41:05" date="1378060865117" type="2" name="Nick Powell" new="1" dur="143" />
            <log number="07989385391" time="1 Sep 2013 13:41:23" date="1378039283149" type="1" name="Anne Barton" new="1" dur="19" />
            <log number="+447943549963" time="1 Sep 2013 13:26:31" date="1378038391562" type="2" name="John M Barton" new="1" dur="0" />
            <log number="+447943549963" time="1 Sep 2013 13:11:46" date="1378037506896" type="2" name="John M Barton" new="1" dur="0" />

    </alllogs>"""


    def __init__(self,
                 writer,
                 ignore_incoming,
                 ignore_outgoing,
                 ignore_missed,
                 ignore_voicemail,
                 ignore_rejected,
                 ignore_refused,
                 minimum_duration
                 ):
        """
        Ctor

        @param writer: orgwriter
        @param ignore_incoming:  ignore incoming phonecalls
        @param ignore_outgoing:  ignore outgoing phonecalls
        @param ignore_missed:    ignore missed   phonecalls
        @param ignore_voicemail: ignore voicemail phonecalls
        @param ignore_rejected:  ignore rejected phonecalls
        @param ignore_refused:   ignore refused  phonecalls
        @param minimum_duration:    ignore phonecalls less than that time
        """
        self._writer = writer
        self._ignore_incoming = ignore_incoming
        self._ignore_outgoing = ignore_outgoing
        self._ignore_missed = ignore_missed
        self._ignore_voicemail = ignore_voicemail
        self._ignore_rejected = ignore_rejected
        self._ignore_refused = ignore_refused
        self._minimum_duration = minimum_duration

    def startElement(self, name, attrs):
        """
        at every <log> write to orgfile
        """
        logging.debug("Handler @startElement name=%s,attrs=%s", name, attrs)

        if name == "log":
            call_number = attrs['number']
            call_duration = int(attrs['dur'])

            call_date = int(attrs['date']) / 1000     # unix epoch

            call_type = int(attrs['type'])
            call_incoming = call_type == 1
            call_outgoing = call_type == 2
            call_missed = call_type == 3
            call_voicemail = call_type == 4
            call_rejected = call_type == 5
            call_refused = call_type == 6

            call_name = attrs['name']

            output = "Phonecall "

            skip = False

            if call_incoming:
                output += "from "
                if self._ignore_incoming:
                    skip = True
            elif call_outgoing:
                output += "to "
                if self._ignore_outgoing:
                    skip = True
            elif call_missed:
                output += "missed "
                if self._ignore_missed:
                    skip = True
            elif call_voicemail:
                output += "voicemail "
                if self._ignore_voicemail:
                    skip = True
            elif call_rejected:
                output += "rejected "
                if self._ignore_rejected:
                    skip = True
            elif call_refused:
                output += "refused "
                if self._ignore_refused:
                    skip = True
            else:
                raise Exception("Invalid Phonecall Type: %d", call_type)

            call_number_string = ""
            if call_number != "-1":
                call_number_string = call_number
            else:
                call_number_string = "Unknown Number"

            name_string = ""
            if call_name != "(Unknown)":
                name_string = '[[contact:' + call_name + '][' + call_name + ']]'
            else:
                name_string = "Unknown"
            output += name_string

            if call_duration < self._minimum_duration:
                skip = True

            timestamp = OrgFormat.datetime(time.gmtime(call_date))

            end_datetimestamp = datetime.datetime.utcfromtimestamp(call_date + call_duration)
            logging.debug("timestamp[%s] duration[%s] end[%s]" %
                          (str(timestamp), str(call_duration), str(end_datetimestamp)))

            end_timestamp_string = OrgFormat.datetime(end_datetimestamp)
            logging.debug("end_time [%s]" % end_timestamp_string)

            data_for_hashing = output + timestamp
            properties = OrgProperties(data_for_hashing=data_for_hashing)
            properties.add("NUMBER", call_number_string)
            properties.add("DURATION", call_duration)
            properties.add("NAME", call_name)

            if not skip:
                self._writer.write_org_subitem(output=output,
                                               timestamp=timestamp + '-' + end_timestamp_string,
                                               properties=properties
                                               )


class PhonecallsSuperBackupMemacs(Memacs):
    def _parser_add_arguments(self):
        """
        overwritten method of class Memacs

        add additional arguments
        """
        Memacs._parser_add_arguments(self)

        self._parser.add_argument(
            "-f", "--file", dest="smsxmlfile",
            action="store", required=True,
            help="path to sms xml backup file")

        self._parser.add_argument(
            "--ignore-incoming", dest="ignore_incoming",
            action="store_true",
            help="ignore incoming phonecalls")

        self._parser.add_argument(
            "--ignore-outgoing", dest="ignore_outgoing",
            action="store_true",
            help="ignore outgoing phonecalls")

        self._parser.add_argument(
            "--ignore-missed", dest="ignore_missed",
            action="store_true",
            help="ignore outgoing phonecalls")

        self._parser.add_argument(
            "--ignore-voicemail", dest="ignore_voicemail",
            action="store_true",
            help="ignore voicemail phonecalls")

        self._parser.add_argument(
            "--ignore-rejected", dest="ignore_rejected",
            action="store_true",
            help="ignore rejected phonecalls")

        self._parser.add_argument(
            "--ignore-refused", dest="ignore_refused",
            action="store_true",
            help="ignore refused phonecalls")

        self._parser.add_argument(
            "--minimum-duration", dest="minimum_duration",
            action="store", type=int,
            help="[sec] show only calls with duration >= this argument")


    def _parser_parse_args(self):
        """
        overwritten method of class Memacs

        all additional arguments are parsed in here
        """
        Memacs._parser_parse_args(self)
        if not (os.path.exists(self._args.smsxmlfile) or \
                     os.access(self._args.smsxmlfile, os.R_OK)):
            self._parser.error("input file not found or not readable")


    def _main(self):
        """
        gets called automatically from Memacs class.
        read the lines from phonecalls backup xml file,
        parse and write them to org file
        """

        data = CommonReader.get_data_from_file(self._args.smsxmlfile)

        try:
            xml.sax.parseString(data.encode('utf-8'),
                                PhonecallsSaxHandler(self._writer,
                                              self._args.ignore_incoming,
                                              self._args.ignore_outgoing,
                                              self._args.ignore_missed,
                                              self._args.ignore_voicemail,
                                              self._args.ignore_rejected,
                                              self._args.ignore_refused,
                                              self._args.minimum_duration,
                                              ))
        except SAXParseException:
            logging.error("No correct XML given")
            sys.exit(1)
